"""
   Copyright 2016 University of Auckland

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""
from opencmiss.argon.argonsceneviewer import ArgonSceneviewer

LAYOUT1 = {
    "Name": "Layout1",
    "Scenes": [
        {
            "Row": 0,
            "Col": 0,
            "Sceneviewer": {},
        }
    ]
}

LAYOUT2x2GRID = {
    "Name": "Layout2x2Grid",
    "GridSpecification": {
        "NumRows": 2,
        "NumCols": 2,
        "Left": None,
        "Bottom": None,
        "Right": None,
        "Top": None,
        "WidthSpace": None,
        "HeightSpace": None,
        "WidthRatios": None,
        "HeightRatios": None
    },
    "Scenes": [
        {
            "Row": 1,
            "Col": 1,
            "Sceneviewer": {},
        },
        {
            "Row": 0,
            "Col": 1,
            "Sceneviewer": {},
        },
        {
            "Row": 1,
            "Col": 0,
            "Sceneviewer": {},
        },
        {
            "Row": 0,
            "Col": 0,
            "Sceneviewer": {},
        }
    ]
}

LAYOUTS = [
    LAYOUT1,
    LAYOUT2x2GRID,
]


class ArgonViewManager(object):
    """
    Manages and serializes ArgonViews.
    """

    def __init__(self, zincContext):
        self._zincContext = zincContext
        self._views = []
        self._activeView = None

    def getZincContext(self):
        """
        Returns the underlying Zinc context for the view manager.

        :return: opencmiss.zinc.context.Context
        """
        return self._zincContext

    def deserialize(self, d):
        """
        Read the JSON description to the ArgonViewManager object. This will change settings of ArgonViewManager Object.

        :param  d: Python JSON object containing the JSON description of ArgonViewManager object.
        """
        self._activeView = d["ActiveView"] if "ActiveView" in d else None
        if "Children" in d:
            for i in d["Children"]:
                view = ArgonView(self._zincContext)
                view.deserialize(i)
                self._views.append(view)

    def serialize(self):
        """
        Write the JSON file describing the ArgonViewManager, which can be used to store the current ArgonViewManager settings.

        :return: Python JSON object containing the JSON description of ArgonViewManager object.
        """
        dictOutput = {}
        if self._activeView:
            dictOutput["ActiveView"] = self._activeView
        dictOutput["Children"] = []
        if self._views:
            tmpOutput = []
            for view in self._views:
                tmpOutput.append(view.serialize())
            dictOutput["Children"] = tmpOutput
        return dictOutput

    def getActiveView(self):
        """
        Returns current active view.

        :return: ArgonView
        """
        return self._activeView

    def setActiveView(self, view):
        """
        Sets current active view.

        :param view: ArgonView
        """
        self._activeView = view

    def getView(self, index):
        """
        Get view by index.

        :param index: int
        """
        return self._views[index]

    def getViews(self):
        """
        Get a list of views.

        :return: list
        """
        return self._views

    def viewCount(self):
        """
        Get the total number of views in this ArgonViewManager object.

        :return: int
        """
        return len(self._views)

    def setViews(self, views):
        """
        Set views.

        :param views: list
        """
        self._views = views

    def addViewByType(self, view_type, name=None):
        """
        Create a new view with given view layout type, there are two avaiable types:
        LAYOUT1 for view with single sceneviewer, LAYOUT2x2GRID for view with four sceneviewers.

        :param view_type: View layout type.
        :param name: Name of the new view, default is None. If it is None, will use view type as name.
        """
        for layout in LAYOUTS:
            if layout["Name"] == view_type:
                new_view = ArgonView(self._zincContext)
                new_view.deserialize(layout)
                if name is None:
                    name = view_type
                if self._name_in_use(name):
                    name = self._next_available_name(name)
                new_view.setName(name)
                self._views.append(new_view)
                return new_view

    def removeView(self, identifier):
        """
        Remove view by given identifier.

        :param identifier: The identifier of view.
        """
        del self._views[identifier]

    def _name_in_use(self, name):
        for view in self._views:
            if view.getName() == name:
                return True

        return False

    def _next_available_name(self, name_stem):
        iteration = 1
        while self._name_in_use(f"{name_stem}_{iteration}"):
            iteration += 1

        return f"{name_stem}_{iteration}"

    def updateSceneviewers(self, view_index, sceneviewers_info):
        """
        Update sceneviewer info in the view.

        :param view_index: The index of view that need to be updated.
        :param sceneviewers_info: New sceneviewers info.
        """
        if 0 <= view_index < len(self._views):
            view = self._views[view_index]
            for sceneviewer_info in sceneviewers_info:
                view.updateSceneviewer(sceneviewer_info["Row"], sceneviewer_info["Col"], sceneviewer_info["Sceneviewer"])


class ArgonView(object):
    """
    Defines and serializes a single view.
    """

    def __init__(self, zincContext):
        self._zincContext = zincContext
        self._name = None
        self._gridSpecification = None
        self._scenes = []

    def getZincContext(self):
        """
        Return the zinc Context of current argon view.

        :return: opencmiss.zinc.context.Context
        """
        return self._zincContext

    def deserialize(self, d):
        """
        Read the JSON description to the Argon View object. This will change settings of ArgonView Object.

        :param  d: Python JSON object containing the JSON description of Argon view object.
        """
        self._name = d["Name"] if "Name" in d else None
        self._gridSpecification = d["GridSpecification"] if "GridSpecification" in d else None

        if "Scenes" in d:
            for s in d["Scenes"]:
                scene = {"Row": 0, "Col": 0}
                if "Row" in s:
                    scene["Row"] = s["Row"]
                if "Col" in s:
                    scene["Col"] = s["Col"]
                if "Sceneviewer" in s:
                    sceneviewer = ArgonSceneviewer(self._zincContext)
                    sceneviewer.deserialize(s["Sceneviewer"])
                    scene["Sceneviewer"] = sceneviewer

                self._scenes.append(scene)

    def serialize(self):
        """
        Write the JSON file describing the Argon view, which can be used to store the current Argon view settings.

        :return: Python JSON object containing the JSON description of Argon view object.
        """
        dictOutput = {}
        if self._name:
            dictOutput["Name"] = self._name
        if self._gridSpecification:
            dictOutput["GridSpecification"] = self._gridSpecification
        dictOutput["Scenes"] = []
        if self._scenes:
            for scene in self._scenes:
                tmp_output = {
                    "Row": scene["Row"],
                    "Col": scene["Col"],
                    "Sceneviewer": scene["Sceneviewer"].serialize(),
                }
                dictOutput["Scenes"].append(tmp_output)
        return dictOutput

    def getName(self):
        """
        Returns the name of current view.

        :return: string
        """
        return self._name

    def setName(self, name):
        """
        Set name of current view.

        :param name: string
        """
        self._name = name

    def getScenes(self):
        """
        Returns a list of scene in current view.

        :return: list
        """
        return self._scenes

    def getGridSpecification(self):
        """
        Returns the grid specification of current region.

        :return: string
        """
        return self._gridSpecification

    def updateSceneviewer(self, row, col, sceneviewer):
        """
        Update sceneviewer at given grid location.

        :param row: int
        :param col: int
        :param sceneviewer: ArgonSceneviewer
        """
        for scene in self._scenes:
            if scene["Row"] == row and scene["Col"] == col:
                scene["Sceneviewer"].updateParameters(sceneviewer)



"""
   Copyright 2016 University of Auckland

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""
import json

from opencmiss.zinc.status import OK as ZINC_OK
from opencmiss.argon.argonerror import ArgonError

SPECTRUM_GLYPH_NAME_FORMAT = 'colour_bar_{0}'


class ArgonSpectrums(object):
    """
    Manages and serializes Zinc Spectrums within Neon.
    Generates colour bar glyphs for spectrums, which is automatically done if if not found on loading.
    """

    def __init__(self, zincContext):
        self._zincContext = zincContext
        self._spectrummodule = zincContext.getSpectrummodule()
        self._findOrCreateAllSpectrumGlyphColourBars()

    def getZincContext(self):
        """
        Return the zinc Context of current argon spectrum.

        :return: opencmiss.zinc.context.Context
        """
        return self._zincContext

    def deserialize(self, dictInput):
        """
        Read the JSON description to the Argon spectrum object. This will change the spectrums in the Zinc spectrum module. 
        Raising an ArgonError if Zinc spectrum module read JSON description failed.

        :param  dictInput: The string containing JSON description
        """
        spectrumsDescription = json.dumps(dictInput)
        result = self._spectrummodule.readDescription(spectrumsDescription)
        if result != ZINC_OK:
            raise ArgonError("Failed to read spectrums")
        self._findOrCreateAllSpectrumGlyphColourBars()

    def serialize(self):
        """
        Write the JSON file describing the spectrums in the argon spectrum object, which can be used to store the current spectrum settings.

        :return: Python JSON object containing the JSON description of argon spectrum object, otherwise 0.
        """
        spectrumsDescription = self._spectrummodule.writeDescription()
        dictOutput = json.loads(spectrumsDescription)
        return dictOutput

    def _findOrCreateAllSpectrumGlyphColourBars(self):
        """
        Ensures there exists a colour bar for each spectrum.
        """
        iterator = self._spectrummodule.createSpectrumiterator()
        spectrum = iterator.next()
        while spectrum.isValid():
            self.findOrCreateSpectrumGlyphColourBar(spectrum)
            spectrum = iterator.next()

    def _findSpectrumGlyphColourBar(self, spectrum):
        """
        Find or create a GlyphColourBar for spectrum in the glyph module.
        Newly created colour bar is set up for display in the normalised window coordinates at left.
        """
        glyphmodule = self._zincContext.getGlyphmodule()
        glyphName = SPECTRUM_GLYPH_NAME_FORMAT.format(spectrum.getName())

        # try to find by name first
        glyph = glyphmodule.findGlyphByName(glyphName)
        colourBar = glyph.castColourBar()
        if colourBar.isValid() and (colourBar.getSpectrum() == spectrum):
            return colourBar

        # attempt to find by matching spectrum but do not rename it
        glyphiterator = glyphmodule.createGlyphiterator()
        glyph = glyphiterator.next()
        while glyph.isValid():
            colourBar = glyph.castColourBar()
            if colourBar.isValid() and (colourBar.getSpectrum() == spectrum):
                return colourBar
            glyph = glyphiterator.next()

        return None

    def findOrCreateSpectrumGlyphColourBar(self, spectrum):
        """
        Find or create a GlyphColourBar for spectrum in the glyph module.
        Newly created colour bar is set up for display in the normalised window coordinates at left.
        
        :param spectrum: The spectrum that the colour bar belongs to.
        """
        colourBar = self._findSpectrumGlyphColourBar(spectrum)
        if colourBar:
            return colourBar

        glyphmodule = self._zincContext.getGlyphmodule()
        glyphName = SPECTRUM_GLYPH_NAME_FORMAT.format(spectrum.getName())

        # create a new colour bar, matching Cmgui's defaults:
        glyphmodule.beginChange()
        colourBar = glyphmodule.createGlyphColourBar(spectrum)
        tmpName = glyphName
        i = 1
        while (colourBar.setName(tmpName) != ZINC_OK):
            tmpName = glyphName + str(i)
            i += 1
        colourBar.setManaged(True)
        colourBar.setCentre([-0.9, 0.0, 0.5])
        colourBar.setAxis([0.0, 1.6, 0.0])  # includes length
        colourBar.setSideAxis([0.06, 0.0, 0.0])  # includes radius
        colourBar.setExtendLength(0.06)
        colourBar.setTickLength(0.04)
        colourBar.setLabelDivisions(10)
        colourBar.setNumberFormat('%+.4e')
        glyphmodule.endChange()
        return colourBar

    def renameSpectrum(self, spectrum, name):
        """
        Renames spectrum and its glyph

        :param spectrum: The spectrum that need to be renamed.
        :param name: New name to the spectrum.
        :return: True on success, otherwise False (means name not set)
        """
        colourBar = self.findOrCreateSpectrumGlyphColourBar(spectrum)
        result = spectrum.setName(name)
        if result == ZINC_OK:
            glyphName = SPECTRUM_GLYPH_NAME_FORMAT.format(name)
            tmpName = glyphName
            i = 1
            while (colourBar.setName(tmpName) != ZINC_OK):
                tmpName = glyphName + str(i)
                i += 1
            return True
        return False

    def removeSpectrumByName(self, name):
        """
        Unmanages spectrum and its colour bar. Note spectrum is only removed if neither are in use.
        
        :return: True if spectrum and colour bar removed, false if failed i.e. either are in use.
        """
        spectrum = self._spectrummodule.findSpectrumByName(name)
        colourBar = self._findSpectrumGlyphColourBar(spectrum)
        if colourBar:
            colourBarName = colourBar.getName()
            colourBar.setManaged(False)
            del colourBar
            colourBar = self._findSpectrumGlyphColourBar(spectrum)
            if colourBar and (colourBar.getName() == colourBarName):
                # colour bar is in use; do not remove spectrum
                colourBar.setManaged(True)
                return False
        spectrum.setManaged(False)
        del spectrum
        spectrum = self._spectrummodule.findSpectrumByName(name)
        if spectrum.isValid():
            # spectrum is in use so can't remove
            spectrum.setManaged(True)
            self.findOrCreateSpectrumGlyphColourBar(spectrum)
            return False
        return True



"""
   Copyright 2015 University of Auckland

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""
import json

from packaging import version

from opencmiss.argon import __version__
from opencmiss.argon.argonregion import ArgonRegion, REGION_PATH_SEPARATOR
from opencmiss.argon.argonspectrums import ArgonSpectrums
from opencmiss.argon.argonmaterials import ArgonMaterials
from opencmiss.argon.argonviews import ArgonViewManager
from opencmiss.argon.argontessellations import ArgonTessellations
from opencmiss.argon.argonerror import ArgonError
from opencmiss.argon.argonlogger import ArgonLogger
from opencmiss.argon.settings import mainsettings

from opencmiss.zinc.context import Context
from opencmiss.zinc.material import Material


class ArgonDocument(object):

    def __init__(self, name="Argon"):
        self._name = name
        self._zincContext = None
        self._rootRegion = None
        self._spectrums = None
        self._materials = None
        self._view_manager = None
        self._tessellations = None

    def checkVersion(self, minimum_required):
        """
        Check the version number of this Argon library. Raise an error if current Argon is less than the minimun required version.
        """
        if version.parse(__version__) < version.parse(minimum_required):
            raise SyntaxError(f"Argon document error - Argon document must be at least version '{minimum_required}'.")

    def getName(self):
        """
        Returns the name of the zinc Context.

        :return: string
        """
        return self._zincContext.getName()

    def initialiseVisualisationContents(self):
        """
        Initialise the default Visualisation Contents.
        """
        self._zincContext = Context(self._name)

        sceneviewermodule = self._zincContext.getSceneviewermodule()
        sceneviewermodule.setDefaultBackgroundColourRGB([1.0, 1.0, 1.0])

        # set up standard materials and glyphs
        materialmodule = self._zincContext.getMaterialmodule()
        materialmodule.beginChange()
        materialmodule.defineStandardMaterials()
        # make default material black
        defaultMaterial = materialmodule.getDefaultMaterial()
        defaultMaterial.setAttributeReal3(Material.ATTRIBUTE_AMBIENT, [0.0, 0.0, 0.0])
        defaultMaterial.setAttributeReal3(Material.ATTRIBUTE_DIFFUSE, [0.0, 0.0, 0.0])
        # still want surfaces to default to white material
        white = materialmodule.findMaterialByName("white")
        materialmodule.setDefaultSurfaceMaterial(white)
        materialmodule.endChange()
        glyphmodule = self._zincContext.getGlyphmodule()
        glyphmodule.defineStandardGlyphs()

        zincRootRegion = self._zincContext.getDefaultRegion()
        self._rootRegion = ArgonRegion(name=None, zincRegion=zincRootRegion, parent=None)
        self._rootRegion.connectRegionChange(self._regionChange)

        self._materials = materialmodule
        self._spectrums = ArgonSpectrums(self._zincContext)
        self._materials = ArgonMaterials(self._zincContext)
        self._view_manager = ArgonViewManager(self._zincContext)
        self._tessellations = ArgonTessellations(self._zincContext)
        ArgonLogger.setZincContext(self._zincContext)

    def freeVisualisationContents(self):
        """
        Deletes subobjects of document to help free memory held by Zinc objects earlier.
        """
        self._rootRegion.freeContents()
        del self._tessellations
        del self._spectrums
        del self._materials
        del self._view_manager
        del self._rootRegion
        del self._zincContext

    def _regionChange(self, changedRegion, treeChange):
        """
        If root region has changed, set its new Zinc region as Zinc context's default region.
        :param changedRegion: The top region changed
        :param treeChange: True if structure of tree, or zinc objects reconstructed
        """
        if treeChange and (changedRegion is self._rootRegion):
            zincRootRegion = changedRegion.getZincRegion()
            self._zincContext.setDefaultRegion(zincRootRegion)

    def deserialize(self, state):
        """
        Read the JSON description to the argon document object. This will change the settings of ArgonDocument Object.

        :param  state: string serialisation of Argon JSON document.
        """
        d = json.loads(state)
        if not (("OpenCMISS-Argon Version" in d) and ("RootRegion" in d)):
            raise ArgonError("Invalid Argon document")

        document_version = d["OpenCMISS-Argon Version"]
        document_version_string = ".".join(document_version)
        if version.parse(document_version_string) > version.parse(mainsettings.VERSION_STRING):
            raise ArgonError(f"Document version '{document_version_string}' is greater than this version of Argon ({mainsettings.VERSION_STRING})."
                             f" Please update your Argon application.")
        # Ideally would enclose following in:
        # try: zincRegion.beginHierarchicalChange() ... finally: zincRegion.endHierarchicalChange()
        # Can't do this due to Zinc issue 3924 which prevents computed field wrappers being created, so graphics can't find fields
        if "Tessellations" in d:
            self._tessellations.deserialize(d["Tessellations"])
        if "Spectrums" in d:
            self._spectrums.deserialize(d["Spectrums"])
        if "Materials" in d:
            self._materials.deserialize(d["Materials"])
        if "Views" in d:
            self._view_manager.deserialize(d["Views"])
        self._rootRegion.deserialize(d["RootRegion"])

    def serialize(self, base_path=None):
        """
        Write the JSON file describing the Argon document in the argon document object, which can be used to store the current argon document settings.

        :param base_path: The base path of JSON file, default is None.
        :return: Python JSON object containing the JSON description of argon document object.
        """
        dictOutput = {
            "OpenCMISS-Argon Version": mainsettings.VERSION_LIST,
            "Spectrums": self._spectrums.serialize(),
            "Materials": self._materials.serialize(),
            "Views": self._view_manager.serialize(),
            "Tessellations": self._tessellations.serialize(),
            "RootRegion": self._rootRegion.serialize(base_path)
        }
        return json.dumps(dictOutput, default=lambda o: o.__dict__, sort_keys=True, indent=2)

    def getZincContext(self):
        """
        Returns the underlying Zinc context for the document.

        :return: opencmiss.zinc.context.Context
        """
        return self._zincContext

    def getRootRegion(self):
        """
        Returns the root region in the context. A convenience for applications that need only one region tree.

        :return: ArgonRegion
        """
        return self._rootRegion

    def getSpectrums(self):
        """
        Returns the spectrums in the context.

        :return: ArgonSpectrums
        """
        return self._spectrums

    def getMaterials(self):
        """
        Returns the materials in the context.

        :return: ArgonMaterials
        """
        return self._materials

    def getViewManager(self):
        """
        Returns the view manager in the context.

        :return: ArgonViewManager
        """
        return self._view_manager

    def getTessellations(self):
        """
        Returns the tessellations in the context.

        :return: ArgonTessellations
        """
        return self._tessellations

    def findRegion(self, name):
        """
        Find region by name.

        :param name: The name of the region as a string.
        :return: ArgonRegion or None if region is not found.
        """
        if not name.endswith(REGION_PATH_SEPARATOR):
            name += REGION_PATH_SEPARATOR
        return _findSubRegion(self._rootRegion, name)


def _findSubRegion(region, name):
    region_path = region.getPath()
    if name == region_path:
        return region
    elif name.startswith(region_path):
        for child_index in range(region.getChildCount()):
            result = _findSubRegion(region.getChild(child_index), name)
            if result is not None:
                return result

    return None



"""
   Copyright 2015 University of Auckland

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""
import os
import pathlib

from opencmiss.zinc.streamregion import StreaminformationRegion
from opencmiss.argon.argonerror import ArgonError


def _file_name_to_relative_path(file_name, base_path):
    if (base_path is None) or (not os.path.isabs(file_name)) or (os.path.commonprefix([file_name, base_path]) == ""):
        return pathlib.PurePath(file_name).as_posix()
    return pathlib.PurePath(os.path.relpath(file_name, base_path)).as_posix()


class ArgonModelSourceFile(object):

    def __init__(self, file_name=None, dict_input=None):
        self._time = None
        self._format = None
        self._edit = False
        self._loaded = False
        if file_name is not None:
            self._file_name = file_name
        else:
            self._deserialize(dict_input)

    def getType(self):
        """
        Returns Argon Model Source File type "FILE".

        :return: string
        """
        return "FILE"

    def addToZincStreaminformationRegion(self, stream_info):
        """
        Add to Zinc Stream information Region.

        :param stream_info: streamInfo
        """
        if self._edit:
            return
        if not self._file_name:
            self._edit = True
            return
        resource = stream_info.createStreamresourceFile(self._file_name)
        self._loaded = True
        if self._time is not None and self._time:
            time = self._time
            if not isinstance(self._time, float):
                time = float(self._time)
            stream_info.setResourceAttributeReal(resource, StreaminformationRegion.ATTRIBUTE_TIME, time)
        # if self._format is not None:
        #    if format == "EX":
        #        #can't set per-resource file format
        #        #streamInfo.setResourceFileFormat(resource, StreaminformationRegion.FILE_FORMAT_EX)

    def unload(self):
        """
        Set Argon model sources loaded state to False.
        """
        self._loaded = False

    def getFileName(self):
        """
        Returns the file name of current Argon model sources.

        :return: string
        """
        return self._file_name

    def setFileName(self, file_name):
        """
        Set file name.

        :param file_name: string
        """
        self._file_name = file_name

    def getTime(self):
        """
        Returns the time of current Argon model sources.

        :return: string
        """
        return self._time

    def setTime(self, time):
        """
        Change time.

        :param time: string
        """
        self._time = time

    def getDisplayName(self):
        """
        Returns the display name of current Argon model sources.

        :return: string
        """
        editText = "[To Apply] " if self._edit else ""
        if self._time is None:
            timeText = ""
        else:
            timeText = ", time " + repr(self._time)
        displayFileName = os.path.basename(self._file_name)
        return editText + "File " + displayFileName + timeText

    def isLoaded(self):
        """
        Returns the load state of current Argon model sources.

        :return: boolean
        """
        return self._loaded

    def isEdit(self):
        """
        Returns the edit state of current Argon model sources.

        :return: boolean
        """
        return self._edit

    def setEdit(self, edit):
        """
        Change edit state for Argon model sources.

        :param edit: boolean
        """
        self._edit = edit

    def _deserialize(self, dict_input):
        # convert to absolute file path so can save Neon file to new location and get correct relative path
        self._file_name = os.path.abspath(dict_input["FileName"])
        if "Edit" in dict_input:
            self._edit = dict_input["Edit"]
        if "Format" in dict_input:
            self._format = dict_input["Format"]
        if "Time" in dict_input:
            self._time = dict_input["Time"]

    def serialize(self, base_path=None):
        """
        Write the JSON file describing the Argon model sources, which can be used to store the current Argon model sources settings.
        :param base_path: The base path of JSON file, default is None.
        :return: Python JSON object containing the JSON description of Argon model sources object.
        """
        dict_output = {
            "Type": self.getType(),
            "FileName": _file_name_to_relative_path(self._file_name, base_path)
        }
        if self._edit:
            dict_output["Edit"] = True
        if self._format is not None:
            dict_output["Format"] = self._format
        if self._time is not None:
            dict_output["Time"] = self._time

        return dict_output


def deserializeArgonModelSource(dict_input):
    """
    Factory method for creating the appropriate neon model source type from the dict serialization
    """
    if "Type" not in dict_input:
        raise ArgonError("Model source is missing Type attribute")

    typeString = dict_input["Type"]
    if typeString == "FILE":
        modelSource = ArgonModelSourceFile(dict_input=dict_input)
    else:
        raise ArgonError("Model source has unrecognised Type " + typeString)
    return modelSource



"""
   Copyright 2016 University of Auckland

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""
import json

from opencmiss.zinc.status import OK as ZINC_OK
from opencmiss.argon.argonerror import ArgonError


class ArgonMaterials(object):
    """
    Manages and serializes Zinc Materials.
    """

    def __init__(self, zincContext):
        self._zincContext = zincContext
        self._materialsmodule = zincContext.getMaterialmodule()

    def getZincContext(self):
        """
        Returns the underlying Zinc context for the Argon Material.

        :return: opencmiss.zinc.context.Context
        """
        return self._zincContext

    def deserialize(self, dictInput):
        """
        Read the JSON description to the argon Material object. This will change the materials in the material module.

        :param dictInput: The string containing JSON description.
        """
        materialsDescription = json.dumps(dictInput)
        result = self._materialsmodule.readDescription(materialsDescription)
        if result != ZINC_OK:
            raise ArgonError("Failed to read materials")

    def serialize(self):
        """
        Write the JSON file describing the materials in the Argon Materials object, which can be used to store the current material settings.

        :return: Python JSON object containing the JSON description of Argon Materials object, otherwise 0.
        """
        materialsDescription = self._materialsmodule.writeDescription()
        dictOutput = json.loads(materialsDescription)
        return dictOutput
